

function [ model ] = RMCRMIL_TrainHyperplane( bags, bagLabels, instancesBagIdx, params )
%RMCRMIL_TrainHyperplane Train Regularized Multi-Concept MIL model using concepts modelled as hyperplanes in the instance space. 
%The implementation follows the method presented in: http://cmtech.upf.edu/sites/default/files/project-files/RMCMIL_BMVC2014.pdf
%   Call:
%       model = RMCRMIL_TrainHyperplane( bags, bagLabels, instancesBagIdx, params ) returns the parameters of the RMCMIL model
%   Input:
%       bags: NxD Matrix with N samples of D dimensions contained in all the bags
%
%       bagLabels: Mx1 vector containing the bag classes (Binary variable 0 or 1)
%
%       instancesBagIdx: Nx1 vector indicating what bag the instance n
%       belongs to. Contains integers from 1 to M, where M is the total
%       number of bags
%       
%       params: Struct with RMCMIL parameters
%           r: Square Root of the generalized mean function parameter(1=artihmetic mean / larger=approximates max function)
%           k: Num of concepts
%           beta:  Parameter of the L21-norm regularization
%   Output:
%       model: Struct with trained model parameters
%       
    [bags model.mean model.std]= zscore(bags); %Features standarization 
    model.r = params.r;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% Params Initialization
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    instancesBagIdxG = ot_idx2G(instancesBagIdx',max(instancesBagIdx));
    Z =  randn(size(bags,2)+1,params.k);
    W = randn(params.k+1,1);
    bags = [bags ones(size(bags,1),1)];

    display('RMCMIL-Optimization')
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%% Optimization Loss function definition 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    options.maxIter = 500;    
    p = [W(:)' Z(:)']';
    funObj = @(p)RMCMIL_Loss(p,bags, bagLabels',instancesBagIdxG,params.k,params.r);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%% Optimization Projection function definition 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    groups = [repmat([1:size(bags,2)-1]',1,params.k);zeros(1,params.k)];
    groups = groups(:);
    funProj = @(p)RMCMIL_Projection(p,params.beta,groups);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%% Optimization with PQN 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    p(:) = minConf_PQN(funObj,p,funProj,options);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%% Learned model 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
    p=p';
    W = p(1:params.k+1)';
    z = p(params.k+2:end)';
    Z = reshape(z,size(bags,2),params.k);
    model.W = W;
    model.Z = Z;
end

